/*
 * Copyright  2004 David Reveman, Peter Nilsson
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the names of
 * David Reveman and Peter Nilsson not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission. David Reveman and Peter Nilsson
 * makes no representations about the suitability of this software for
 * any purpose. It is provided "as is" without express or implied warranty.
 *
 * DAVID REVEMAN AND PETER NILSSON DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DAVID REVEMAN AND
 * PETER NILSSON BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Authors: David Reveman <c99drn@cs.umu.se>
 *          Peter Nilsson <c99pnn@cs.umu.se>
 */

#ifndef GLC_H_INCLUDED
#define GLC_H_INCLUDED

#include <glc-features.h>

#if defined(__SVR4) && defined(__sun)
#  include <sys/int_types.h>
#else
#  if defined(__OpenBSD__)
#    include <inttypes.h>
#  else
#    include <stdint.h>
#  endif
#endif

/* NOTE: Must be manually synchronized with LIBGLC_VERSION in configure.in */
#define GLC_MAJOR 0
#define GLC_MINOR 1
#define GLC_REVISION 0

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

typedef int glc_bool_t;
typedef int glc_fixed16_16_t;

typedef struct _glc_point_fixed_t {
  glc_fixed16_16_t x, y;
} glc_point_fixed_t;

typedef struct _glc_line_fixed_t {
  glc_point_fixed_t p1, p2;
} glc_line_fixed_t;

typedef struct _glc_rectangle_t {
  short x, y;
  unsigned short width, height;
} glc_rectangle_t;

typedef struct _glc_triangle_t {
  glc_point_fixed_t p1, p2, p3;
} glc_triangle_t;

typedef struct _glc_trapezoid_t {
  glc_fixed16_16_t top, bottom;
  glc_line_fixed_t left, right;
} glc_trapezoid_t;

typedef struct _glc_transform_t {
  glc_fixed16_16_t matrix[3][3];
} glc_transform_t;

typedef struct _glc_convolution_t {
  glc_fixed16_16_t matrix[3][3];
} glc_convolution_t;

typedef struct {
  unsigned short red;
  unsigned short green;
  unsigned short blue;
  unsigned short alpha;
} glc_color_t;
  
typedef struct _glc_colorspan_t {
  glc_fixed16_16_t left, right, y;
  glc_color_t left_color;
  glc_color_t right_color;
} glc_colorspan_t;

typedef struct _glc_colorpoint_t {
  glc_point_fixed_t point;
  glc_color_t color;
} glc_colorpoint_t;

typedef struct _glc_colorstop_t {
  glc_fixed16_16_t stop;
  glc_color_t color;
} glc_colorstop_t;
  
typedef struct _glc_color_trapezoid_t {
  glc_colorspan_t top, bottom;
} glc_color_trapezoid_t;

typedef struct _glc_color_triangle_t {
  glc_colorpoint_t p1, p2, p3;
} glc_color_triangle_t;

typedef enum {
  GLC_FILTER_FAST,
  GLC_FILTER_GOOD,
  GLC_FILTER_BEST,
  GLC_FILTER_NEAREST,
  GLC_FILTER_BILINEAR
} glc_filter_t;

typedef enum {
  GLC_OPERATOR_CLEAR,
  GLC_OPERATOR_SRC,
  GLC_OPERATOR_DST,
  GLC_OPERATOR_OVER,
  GLC_OPERATOR_OVER_REVERSE,
  GLC_OPERATOR_IN,
  GLC_OPERATOR_IN_REVERSE,
  GLC_OPERATOR_OUT,
  GLC_OPERATOR_OUT_REVERSE,
  GLC_OPERATOR_ATOP,
  GLC_OPERATOR_ATOP_REVERSE,
  GLC_OPERATOR_XOR,
  GLC_OPERATOR_ADD,
  GLC_OPERATOR_SATURATE
} glc_operator_t;

#define GLC_FEATURE_OFFSCREEN_DRAWING_MASK     (1L << 0)
#define GLC_FEATURE_CONVOLUTION_FILTER_MASK    (1L << 1)
#define GLC_FEATURE_GRADIENT_MASK              (1L << 2)
#define GLC_FEATURE_TEXTURE_RECTANGLE_MASK     (1L << 3)
#define GLC_FEATURE_TEXTURE_NPOT_MASK          (1L << 4)
#define GLC_FEATURE_MULTISAMPLE_MASK           (1L << 5)
#define GLC_FEATURE_OFFSCREEN_MULTISAMPLE_MASK (1L << 6)
#define GLC_FEATURE_ARB_VERTEX_PROGRAM_MASK    (1L << 7)
#define GLC_FEATURE_ARB_FRAGMENT_PROGRAM_MASK  (1L << 8)
#define GLC_FEATURE_ATI_RENDER_TEXTURE_MASK    (1L << 9)

typedef enum {  
  GLC_STANDARD_ARGB32,
  GLC_STANDARD_RGB24,
  GLC_STANDARD_A8,
  GLC_STANDARD_A1
} glc_format_name_t;
  
#define GLC_FORMAT_ID_MASK                  (1 <<  0)
#define GLC_FORMAT_BPP_MASK                 (1 <<  1)
#define GLC_FORMAT_RED_MASK_MASK            (1 <<  2)
#define GLC_FORMAT_GREEN_MASK_MASK          (1 <<  3)
#define GLC_FORMAT_BLUE_MASK_MASK           (1 <<  4)
#define GLC_FORMAT_ALPHA_MASK_MASK          (1 <<  5)
#define GLC_FORMAT_RED_SIZE_MASK            (1 <<  6)
#define GLC_FORMAT_GREEN_SIZE_MASK          (1 <<  7)
#define GLC_FORMAT_BLUE_SIZE_MASK           (1 <<  8)
#define GLC_FORMAT_ALPHA_SIZE_MASK          (1 <<  9)
#define GLC_FORMAT_DEPTH_SIZE_MASK          (1 << 10)
#define GLC_FORMAT_DOUBLEBUFFER_MASK        (1 << 11)
#define GLC_FORMAT_ONSCREEN_MASK            (1 << 12)
#define GLC_FORMAT_OFFSCREEN_MASK           (1 << 13)
#define GLC_FORMAT_MULTISAMPLE_MASK         (1 << 14)
#define GLC_FORMAT_MULTISAMPLE_SAMPLES_MASK (1 << 15)

typedef unsigned long int glc_format_id_t;

typedef struct _glc_drawable_type_t {
  glc_bool_t onscreen;
  glc_bool_t offscreen;
} glc_drawable_type_t;

typedef struct _glc_multisample_format_t {
  glc_bool_t supported;
  unsigned short samples;
} glc_multisample_format_t;

typedef struct _glc_format_t {
  glc_format_id_t id;

  /* bpp and mask values specifies the pixel format for read/draw pixels */
  int bpp;
  unsigned long red_mask;
  unsigned long green_mask;
  unsigned long blue_mask;
  unsigned long alpha_mask;
  
  unsigned short red_size;
  unsigned short green_size;
  unsigned short blue_size;
  unsigned short alpha_size;
  unsigned short depth_size;
  
  glc_bool_t doublebuffer;
  glc_drawable_type_t drawable;
  glc_multisample_format_t multisample;
} glc_format_t;

#define GLC_FORMAT_OPTION_DOUBLEBUFFER_MASK   (1 << 0)
#define GLC_FORMAT_OPTION_SINGLEBUFFER_MASK   (1 << 1)
#define GLC_FORMAT_OPTION_ONSCREEN_MASK       (1 << 2)
#define GLC_FORMAT_OPTION_OFFSCREEN_MASK      (1 << 3)
#define GLC_FORMAT_OPTION_MULTISAMPLE_MASK    (1 << 4)
#define GLC_FORMAT_OPTION_NO_MULTISAMPLE_MASK (1 << 5)

/* glc_status.c */
  
typedef enum {
    GLC_STATUS_SUCCESS = 0,
    GLC_STATUS_NO_MEMORY,
    GLC_STATUS_NULL_POINTER,
    GLC_STATUS_BAD_COORDINATE,
    GLC_STATUS_NOT_SUPPORTED,
    GLC_STATUS_INVALID_MATRIX
} glc_status_t;

const char *
glc_status_string (glc_status_t status);

  
/* glc_surface.c */

typedef struct _glc_surface glc_surface_t;

typedef enum {
  GLC_POLYEDGE_SHARP,
  GLC_POLYEDGE_SMOOTH
} glc_polyedge_t;
  
void
glc_surface_destroy (glc_surface_t *surface);

void
glc_surface_set_transform (glc_surface_t *surface,
                           glc_transform_t *transform);

void
glc_surface_set_convolution (glc_surface_t *surface,
                             glc_convolution_t *convolution);

void
glc_surface_set_repeat (glc_surface_t *surface,
                        glc_bool_t repeat);

void
glc_surface_set_filter (glc_surface_t *surface,
                        glc_filter_t filter);

void
glc_surface_set_polyedge (glc_surface_t *surface,
                          glc_polyedge_t polyedge);

int
glc_surface_get_width (glc_surface_t *surface);

int
glc_surface_get_height (glc_surface_t *surface);

void
glc_surface_realize (glc_surface_t *surface);

void
glc_surface_show (glc_surface_t *surface,
                  int x,
                  int y,
                  unsigned int width,
                  unsigned int height);

void
glc_surface_read_pixels (glc_surface_t *surface,
                         int x,
                         int y,
                         unsigned int width,
                         unsigned int height,
                         char *pixels);

void
glc_surface_draw_pixels (glc_surface_t *surface,
                         int x,
                         int y,
                         unsigned int width,
                         unsigned int height,
                         char *pixels);

void
glc_surface_get_gl_texture (glc_surface_t *surface,
                            unsigned int *name,
                            unsigned int *target,
                            double *texcoord_width,
                            double *texcoord_height,
                            glc_bool_t *repeatable);  

void
glc_surface_gl_begin (glc_surface_t *surface);

void
glc_surface_gl_end (glc_surface_t *surface);

glc_status_t
glc_surface_get_status (glc_surface_t *surface);

unsigned long
glc_surface_features (glc_surface_t *surface);

  
/* glc_programmatic.c */

glc_surface_t *
glc_solid_create (glc_color_t *color);
  
glc_surface_t *
glc_linear_gradient_create (glc_colorpoint_t *start,
                            glc_colorpoint_t *end,
                            glc_colorstop_t *stops,
                            int n_stops);

glc_surface_t *
glc_radial_gradient_create (glc_colorpoint_t *start,
                            glc_colorstop_t *stops,
                            int n_stops);
  
  
/* glc_rect.c */

void
glc_fill_rectangle (glc_operator_t op,
                    glc_surface_t *dst,
                    const glc_color_t *color,
                    int x,
                    int y,
                    unsigned int width,
                    unsigned int height);
  
void
glc_fill_rectangles (glc_operator_t op,
                     glc_surface_t *dst,
                     const glc_color_t *color,
                     const glc_rectangle_t *rects,
                     int n_rects);


/* glc_trap.c */

void
glc_composite_trapezoids (glc_operator_t op,
                          glc_surface_t *src,
                          glc_surface_t *dst,
                          int x_src,
                          int y_src,
                          const glc_trapezoid_t *traps,
                          int n_traps);

void
glc_color_trapezoids (glc_operator_t op,
                      glc_surface_t *dst,
                      const glc_color_trapezoid_t *color_traps,
                      int n_color_traps);


/* glc_tri.c */

void
glc_composite_triangles (glc_operator_t op,
                         glc_surface_t *src,
                         glc_surface_t *dst,
                         int x_src,
                         int y_src,
                         const glc_triangle_t *tris,
                         int n_tris);

void
glc_composite_tri_strip (glc_operator_t op,
                         glc_surface_t *src,
                         glc_surface_t *dst,
                         int x_src,
                         int y_src,
                         const glc_point_fixed_t *points,
                         int n_points);
  

void
glc_composite_tri_fan (glc_operator_t op,
                       glc_surface_t *src,
                       glc_surface_t *dst,
                       int x_src,
                       int y_src,
                       const glc_point_fixed_t *points,
                       int n_points);
  
void
glc_color_triangles (glc_operator_t op,
                     glc_surface_t *dst,
                     const glc_color_triangle_t *color_tris,
                     int n_color_tris);
  

/* glc.c */

void
glc_composite (glc_operator_t op,
               glc_surface_t *src,
               glc_surface_t *mask,
               glc_surface_t *dst,
               int x_src,
               int y_src,
               int x_mask,
               int y_mask,
               int x_dst,
               int y_dst,
               int width,
               int height);


#ifdef LIBGLC_HAS_GLX_BACKEND

#include <X11/Xlib.h>
#include <X11/Xutil.h>
  
/* glc_glx_info.c */

void
glc_glx_initialize (Display *display,
                    int screen);  

unsigned long
glc_glx_features (Display *display,
                  int screen);
  

/* glc_glx_format.c */

glc_format_t *
glc_glx_find_format (Display *display,
                     int screen,
                     unsigned long mask,
                     const glc_format_t *templ,
                     int count);
  
glc_format_t *
glc_glx_find_standard_format (Display *display,
                              int screen,
                              unsigned long options,
                              glc_format_name_t format_name);

XVisualInfo *
glc_glx_get_visual_info_from_format (Display *display,
                                     int screen,
                                     glc_format_t *format);
  

/* glc_glx_surface.c */

glc_surface_t *
glc_glx_surface_create (Display *display,
                        int screen,
                        glc_format_t *format,
                        int width,
                        int height);

glc_surface_t *
glc_glx_surface_create_for_window (Display *display,
                                   int screen,
                                   glc_format_t *format,
                                   Window window);

#endif /* LIBGLC_HAS_GLX_BACKEND */

  
#ifdef LIBGLC_HAS_AGL_BACKEND
  
#include <Carbon/Carbon.h>
  
/* glc_agl_info.c */

void
glc_agl_initialize (void);

unsigned long
glc_agl_features (void);
  

/* glc_agl_format.c */

glc_format_t *
glc_agl_find_format (unsigned long mask,
                     const glc_format_t *templ,
                     int count);
  
glc_format_t *
glc_agl_find_standard_format (unsigned long options,
                              glc_format_name_t format_name);

  
/* glc_agl_surface.c */

glc_surface_t *
glc_agl_surface_create (glc_format_t *format,
                        int width,
                        int height);

glc_surface_t *
glc_agl_surface_create_for_window (glc_format_t *format,
                                   WindowRef window);

#endif /* LIBGLC_HAS_AGL_BACKEND */
  
#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif /* GLC_H_INCLUDED */
