/*
 * Copyright  2004 David Reveman, Peter Nilsson
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the names of
 * David Reveman and Peter Nilsson not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission. David Reveman and Peter Nilsson
 * makes no representations about the suitability of this software for
 * any purpose. It is provided "as is" without express or implied warranty.
 *
 * DAVID REVEMAN AND PETER NILSSON DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DAVID REVEMAN AND
 * PETER NILSSON BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Authors: David Reveman <c99drn@cs.umu.se>
 *          Peter Nilsson <c99pnn@cs.umu.se>
 */

#ifdef HAVE_CONFIG_H
#  include "../config.h"
#endif

#include "glcint.h"

#include <OpenGL/glext.h>

glc_gl_proc_address_list_t _glc_agl_gl_proc_address = {
  glActiveTextureARB,
  glMultiTexCoord2dARB,
  glGenProgramsARB,
  glDeleteProgramsARB,
  glProgramStringARB,
  glBindProgramARB,
  glProgramLocalParameter4dARB,
  glGetProgramivARB,
  0
};

static void
_glc_agl_thread_info_init (glc_agl_thread_info_t *thread_info);

#ifdef PTHREADS

#include <pthread.h>
#include <stdlib.h>

/* thread safe */
static int tsd_initialized = 0;
static pthread_key_t info_tsd;

glc_agl_thread_info_t *
glc_agl_thread_info_get (void)
{
  if (!tsd_initialized) {
    glc_agl_thread_info_t *info = (glc_agl_thread_info_t *)
      malloc (sizeof (glc_agl_thread_info_t));
    pthread_key_create (&info_tsd, NULL);
    pthread_setspecific (info_tsd, info);
    tsd_initialized = 1;
    _glc_agl_thread_info_init (info);
    
    return info;
  } else
    return (glc_agl_thread_info_t *) pthread_getspecific (info_tsd);
}

#else

/* not thread safe */
static glc_agl_thread_info_t _thread_info = {
  NULL,
  NULL,
  0,
  NULL,
  0,
  NULL,
  0,
  { (AGLContext) 0, (AGLPixelFormat) 0, 0 },
  0,
  0,
  0,
  0,
  { 0, 0, 0, 0 }
};

glc_agl_thread_info_t *
glc_agl_thread_info_get (void)
{
  if (_thread_info.context_stack == NULL)
    _glc_agl_thread_info_init (&_thread_info);
      
  return &_thread_info;
}

#endif

static void
_glc_agl_thread_info_init (glc_agl_thread_info_t *thread_info)
{
  GLint attrib[] = {
    AGL_RGBA,
    AGL_NO_RECOVERY,
    AGL_NONE
  };

  thread_info->formats = NULL;
  thread_info->format_ids = NULL;
  thread_info->n_formats = 0;
  thread_info->contexts = NULL;
  thread_info->n_contexts = 0;

  memset (&thread_info->programs, 0, sizeof (glc_programs_t));
  
  thread_info->root_context.pixel_format =
    aglChoosePixelFormat (NULL, 0, attrib);
  thread_info->root_context.context =
    aglCreateContext (thread_info->root_context.pixel_format, NULL);

  aglSetCurrentContext (thread_info->root_context.context);
  glPixelStorei (GL_PACK_ALIGNMENT, 4);
  glPixelStorei (GL_UNPACK_ALIGNMENT, 4);
  
  glc_agl_query_extensions (thread_info);
  glc_agl_query_formats (thread_info);

  thread_info->context_stack = malloc (sizeof (glc_agl_context_info_t));
  thread_info->context_stack_size = 1;
  thread_info->context_stack->surface = NULL;
  thread_info->context_stack->constraint = GLC_CN_NONE;
}

void
glc_agl_initialize (void)
{
  glc_agl_thread_info_get ();
}
slim_hidden_def(glc_agl_initialize);

unsigned long
glc_agl_features (void)
{
  glc_agl_thread_info_t *thread_info =
    glc_agl_thread_info_get ();

  return thread_info->feature_mask;
}
slim_hidden_def(glc_agl_features);
