/*
 * Copyright  2004 David Reveman, Peter Nilsson
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the names of
 * David Reveman and Peter Nilsson not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission. David Reveman and Peter Nilsson
 * makes no representations about the suitability of this software for
 * any purpose. It is provided "as is" without express or implied warranty.
 *
 * DAVID REVEMAN AND PETER NILSSON DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DAVID REVEMAN AND
 * PETER NILSSON BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Authors: David Reveman <c99drn@cs.umu.se>
 *          Peter Nilsson <c99pnn@cs.umu.se>
 */

#ifdef HAVE_CONFIG_H
#  include "../config.h"
#endif

#include "glcint.h"

void
glc_set_operator (glc_operator_t op) {
  switch (op) {
  case GLC_OPERATOR_CLEAR:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ZERO, GL_ZERO);
    break;
  case GLC_OPERATOR_SRC:
    glDisable(GL_BLEND);
    break;
  case GLC_OPERATOR_DST:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ZERO, GL_ONE);
    break;
  case GLC_OPERATOR_OVER:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
    break;
  case GLC_OPERATOR_OVER_REVERSE:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ONE_MINUS_DST_ALPHA, GL_ONE);
    break;
  case GLC_OPERATOR_IN:
    glEnable (GL_BLEND);
    glBlendFunc (GL_DST_ALPHA, GL_ZERO);
    break;
  case GLC_OPERATOR_IN_REVERSE:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ZERO, GL_SRC_ALPHA);
    break;
  case GLC_OPERATOR_OUT:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ONE_MINUS_DST_ALPHA, GL_ZERO);
    break;
  case GLC_OPERATOR_OUT_REVERSE:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ZERO, GL_ONE_MINUS_SRC_ALPHA);
    break;
  case GLC_OPERATOR_ATOP:
    glEnable (GL_BLEND);
    glBlendFunc (GL_DST_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    break;
  case GLC_OPERATOR_ATOP_REVERSE:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ONE_MINUS_DST_ALPHA, GL_SRC_ALPHA);
    break;
  case GLC_OPERATOR_XOR:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ONE_MINUS_DST_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    break;
  case GLC_OPERATOR_ADD:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ONE, GL_ONE);
    break;
  case GLC_OPERATOR_SATURATE:
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA_SATURATE, GL_ONE);
    break;
  default:
    glEnable (GL_BLEND);
    glBlendFunc (GL_ZERO, GL_ONE);
    break;
  }
}
