/*
 * Copyright  2004 David Reveman, Peter Nilsson
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the names of
 * David Reveman and Peter Nilsson not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission. David Reveman and Peter Nilsson
 * makes no representations about the suitability of this software for
 * any purpose. It is provided "as is" without express or implied warranty.
 *
 * DAVID REVEMAN AND PETER NILSSON DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DAVID REVEMAN AND
 * PETER NILSSON BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Authors: David Reveman <c99drn@cs.umu.se>
 *          Peter Nilsson <c99pnn@cs.umu.se>
 */

#ifdef HAVE_CONFIG_H
#  include "../config.h"
#endif

#include "glcint.h"

static void
glc_rectangle_bounds (int n_rects,
                      const glc_rectangle_t *rects,
                      glc_region_box_t *box)
{
  box->x1 = rects->x;
  box->x2 = rects->x + rects->width;
  box->y1 = rects->y;
  box->y2 = rects->y + rects->height;
  rects++;
  n_rects--;
    
  while (n_rects-- > 0) {
    if (rects->x < box->x1)
      box->x1 = rects->x;
    else if ((rects->x + rects->width) > box->x2)
      box->x2 = rects->x + rects->width;
    if (rects->y < box->y1)
      box->y1 = rects->y;
    else if ((rects->y + rects->height) > box->y2)
      box->y2 = rects->y + rects->height;
    rects++;
  }
}

void
glc_fill_rectangle (glc_operator_t op,
                    glc_surface_t *dst,
                    const glc_color_t *color,
                    int x,
                    int y,
                    unsigned int width,
                    unsigned int height)
{
  glc_region_box_t bounds;

  bounds.x1 = x;
  bounds.x2 = x + width;
  bounds.y1 = y;
  bounds.y2 = y + height;
  if (bounds.x1 > dst->width || bounds.y1 > dst->height ||
      bounds.x2 < 0 || bounds.y2 < 0)
    return;

  if (!glc_surface_push_current (dst, GLC_CN_SURFACE_DRAWABLE_CURRENT)) {
    glc_surface_pop_current (dst);
    return;
  }

  glShadeModel (GL_FLAT);
  
  glColor4us (color->red, color->green, color->blue, color->alpha);

  glc_set_operator (op);
                   
  glBegin (GL_QUADS);
    
  glVertex2i (bounds.x1, bounds.y1);
  glVertex2i (bounds.x2, bounds.y1);
  glVertex2i (bounds.x2, bounds.y2);
  glVertex2i (bounds.x1, bounds.y2);
    
  glEnd ();

  glc_surface_dirty (dst, &bounds);
  glc_surface_pop_current (dst);
}
slim_hidden_def(glc_fill_rectangle);

void
glc_fill_rectangles (glc_operator_t op,
                     glc_surface_t *dst,
                     const glc_color_t *color,
                     const glc_rectangle_t *rects,
                     int n_rects)
{
  glc_region_box_t bounds;
  
  glc_rectangle_bounds (n_rects, rects, &bounds);
  if (bounds.x1 > dst->width || bounds.y1 > dst->height ||
      bounds.x2 < 0 || bounds.y2 < 0)
    return;

  if (!glc_surface_push_current (dst, GLC_CN_SURFACE_DRAWABLE_CURRENT)) {
    glc_surface_pop_current (dst);
    return;
  }

  glShadeModel (GL_FLAT);

  glColor4us (color->red, color->green, color->blue, color->alpha);

  glc_set_operator (op);
    
  glBegin (GL_QUADS);
  
  for (; n_rects; n_rects--, rects++) {
    glVertex2i (rects->x, rects->y);
    glVertex2i (rects->x + rects->width, rects->y);
    glVertex2i (rects->x + rects->width, rects->y + rects->height);
    glVertex2i (rects->x, rects->y + rects->height);
  }
  
  glEnd ();

  glc_surface_dirty (dst, &bounds);  
  glc_surface_pop_current (dst);
}
slim_hidden_def(glc_fill_rectangles);
