/*
 * Copyright  2004 David Reveman, Peter Nilsson
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the names of
 * David Reveman and Peter Nilsson not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission. David Reveman and Peter Nilsson
 * makes no representations about the suitability of this software for
 * any purpose. It is provided "as is" without express or implied warranty.
 *
 * DAVID REVEMAN AND PETER NILSSON DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DAVID REVEMAN AND
 * PETER NILSSON BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Authors: David Reveman <c99drn@cs.umu.se>
 *          Peter Nilsson <c99pnn@cs.umu.se>
 */

#ifdef HAVE_CONFIG_H
#  include "../config.h"
#endif

#include "glcint.h"

glc_status_t
glc_status_pop_from_mask (long int *mask)
{
  if (*mask & GLC_STATUS_NO_MEMORY_MASK) {
    *mask &= ~GLC_STATUS_NO_MEMORY_MASK;
    return GLC_STATUS_NO_MEMORY;
  } else if (*mask & GLC_STATUS_NULL_POINTER_MASK) {
    *mask &= ~GLC_STATUS_NULL_POINTER_MASK;
    return GLC_STATUS_NULL_POINTER;
  } else if (*mask & GLC_STATUS_BAD_COORDINATE_MASK) {
    *mask &= ~GLC_STATUS_BAD_COORDINATE_MASK;
    return GLC_STATUS_BAD_COORDINATE;
  } else if (*mask & GLC_STATUS_NOT_SUPPORTED_MASK) {
    *mask &= ~GLC_STATUS_NOT_SUPPORTED_MASK;
    return GLC_STATUS_NOT_SUPPORTED;
  } else if (*mask & GLC_STATUS_INVALID_MATRIX_MASK) {
    *mask &= ~GLC_STATUS_INVALID_MATRIX_MASK;
    return GLC_STATUS_INVALID_MATRIX;
  }
  
  return GLC_STATUS_SUCCESS;
}

const char *
glc_status_string (glc_status_t status)
{
  switch (status) {
  case GLC_STATUS_SUCCESS:
    return "success";
  case GLC_STATUS_NO_MEMORY:
    return "out of memory";
  case GLC_STATUS_NULL_POINTER:
    return "NULL pointer";
  case GLC_STATUS_BAD_COORDINATE:
    return "bad coordinate";
  case GLC_STATUS_NOT_SUPPORTED:
    return "not supported";
  case GLC_STATUS_INVALID_MATRIX:
    return "invalid matrix";
  }

  return "<unknown error status>";
}
