/*
 * Copyright  2004 David Reveman, Peter Nilsson
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the names of
 * David Reveman and Peter Nilsson not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission. David Reveman and Peter Nilsson
 * makes no representations about the suitability of this software for
 * any purpose. It is provided "as is" without express or implied warranty.
 *
 * DAVID REVEMAN AND PETER NILSSON DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL DAVID REVEMAN AND
 * PETER NILSSON BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Authors: David Reveman <c99drn@cs.umu.se>
 *          Peter Nilsson <c99pnn@cs.umu.se>
 */

#ifndef GLCINT_H_INCLUDED
#define GLCINT_H_INCLUDED

#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include "glc.h"

#if __GNUC__ >= 3 && defined(__ELF__)
# define slim_hidden_proto(name)	slim_hidden_proto1(name, INT_##name)
# define slim_hidden_def(name)		slim_hidden_def1(name, INT_##name)
# define slim_hidden_proto1(name, internal)				\
  extern __typeof (name) name						\
	__asm__ (slim_hidden_asmname (internal))			\
	__internal_linkage;
# define slim_hidden_def1(name, internal)				\
  extern __typeof (name) EXT_##name __asm__(slim_hidden_asmname(name))	\
	__attribute__((__alias__(slim_hidden_asmname(internal))))
# define slim_hidden_ulp		slim_hidden_ulp1(__USER_LABEL_PREFIX__)
# define slim_hidden_ulp1(x)		slim_hidden_ulp2(x)
# define slim_hidden_ulp2(x)		#x
# define slim_hidden_asmname(name)	slim_hidden_asmname1(name)
# define slim_hidden_asmname1(name)	slim_hidden_ulp #name
#else
# define slim_hidden_proto(name)
# define slim_hidden_def(name)
#endif

#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 3)) && defined(__ELF__)
#define __internal_linkage	__attribute__((__visibility__("hidden")))
#else
#define __internal_linkage
#endif

#ifndef __GNUC__
#define __attribute__(x)
#endif

#define GLC_STATUS_NO_MEMORY_MASK          (1L << 0)
#define GLC_STATUS_NULL_POINTER_MASK       (1L << 1)
#define GLC_STATUS_BAD_DRAWABLE_MASK       (1L << 2)
#define GLC_STATUS_UNRELATED_SURFACES_MASK (1L << 3)
#define GLC_STATUS_BAD_COORDINATE_MASK     (1L << 4)
#define GLC_STATUS_NOT_SUPPORTED_MASK      (1L << 5)
#define GLC_STATUS_INVALID_MATRIX_MASK     (1L << 6)

#define GLC_TEXTURE_TARGET_2D_MASK        (1L << 0)
#define GLC_TEXTURE_TARGET_RECTANGLE_MASK (1L << 1)
#define GLC_TEXTURE_TARGET_NPOT_MASK      (1L << 2)

#define GLC_FORMAT_ALL_EXCEPT_ID_MASK ((1 << 16) - 2)

#ifdef LIBGLC_HAS_AGL_BACKEND
#include <OpenGL/gl.h>
#else
#include <GL/gl.h>
#endif

#include "glc_glext.h"

typedef struct _glc_gl_proc_address_list_t {
  glc_proc_ptr_gl_active_texture_arb_t gl_active_texture_arb;
  glc_proc_ptr_gl_multi_tex_coord_2d_arb_t gl_multi_tex_coord_2d_arb;
  glc_proc_ptr_gl_gen_programs_arb_t gl_gen_programs_arb;
  glc_proc_ptr_gl_delete_programs_arb_t gl_delete_programs_arb;
  glc_proc_ptr_gl_program_string_arb_t gl_program_string_arb;
  glc_proc_ptr_gl_bind_program_arb_t gl_bind_program_arb;
  glc_proc_ptr_gl_program_local_param_4d_arb_t gl_program_local_param_4d_arb;
  glc_proc_ptr_gl_get_program_iv_arb_t gl_get_program_iv_arb;
  glc_bool_t supported;
} glc_gl_proc_address_list_t;

typedef enum {
  GLC_PROGRAMMATIC_SURFACE_SOLID_TYPE = 0,
  GLC_PROGRAMMATIC_SURFACE_LINEAR_GRADIENT_TYPE,
  GLC_PROGRAMMATIC_SURFACE_RADIAL_GRADIENT_TYPE
} glc_programmatic_surface_type_t;

typedef enum {
  GLC_PROGRAM_TYPE_NONE,
  GLC_PROGRAM_TYPE_NOT_SUPPORTED,
  GLC_PROGRAM_TYPE_SRC_CONVOLUTION,
  GLC_PROGRAM_TYPE_SRC_CONVOLUTION_AND_SOLID_MASK,
  GLC_PROGRAM_TYPE_MASK_CONVOLUTION,
  GLC_PROGRAM_TYPE_MASK_CONVOLUTION_AND_SOLID_SRC,
  GLC_PROGRAM_TYPE_SRC_PROGRAMMATIC,
  GLC_PROGRAM_TYPE_MASK_PROGRAMMATIC,
  GLC_PROGRAM_TYPE_SIMPLE
} glc_program_type_t;

#define GLC_PROGRAMMATIC_SURFACE_NUM \
  (GLC_PROGRAMMATIC_SURFACE_RADIAL_GRADIENT_TYPE + 1)

#define GLC_PROGRAM_2DSRC_2DMASK_OFFSET     0
#define GLC_PROGRAM_RECTSRC_2DMASK_OFFSET   1
#define GLC_PROGRAM_2DSRC_RECTMASK_OFFSET   2
#define GLC_PROGRAM_RECTSRC_RECTMASK_OFFSET 3
#define GLC_PROGRAM_NOSRC_2DMASK_OFFSET     4
#define GLC_PROGRAM_NOSRC_RECTMASK_OFFSET   5
#define GLC_PROGRAM_2DSRC_NOMASK_OFFSET     6
#define GLC_PROGRAM_RECTSRC_NOMASK_OFFSET   7
#define GLC_PROGRAM_NOSRC_NOMASK_OFFSET     8

#define GLC_PROGRAM_SRC_OPERATION_OFFSET  0
#define GLC_PROGRAM_MASK_OPERATION_OFFSET 9

#define GLC_VERTEX_PROGRAM_TYPES 2
#define GLC_FRAGMENT_PROGRAM_TYPES 18
#define GLC_FRAGMENT_PROGRAMMATIC_PROGRAM_TYPES \
  (GLC_FRAGMENT_PROGRAM_TYPES * GLC_PROGRAMMATIC_SURFACE_NUM)

typedef struct _glc_programs_t {
  unsigned long vertex_convolution[GLC_VERTEX_PROGRAM_TYPES];
  unsigned long fragment_simple[GLC_FRAGMENT_PROGRAM_TYPES];
  unsigned long fragment_convolution[GLC_FRAGMENT_PROGRAM_TYPES * 3];
  unsigned long fragment_programmatic[GLC_FRAGMENT_PROGRAMMATIC_PROGRAM_TYPES];
} glc_programs_t;

typedef enum {
  GLC_CN_NONE,
  GLC_CN_ANY_CONTEXT_CURRENT,
  GLC_CN_SURFACE_CONTEXT_CURRENT,
  GLC_CN_SURFACE_DRAWABLE_CURRENT
} glc_constraint_t;

typedef struct _glc_region_box_t {
  int x1, x2, y1, y2;
} glc_region_box_t;

typedef struct _glc_sub_pixel_region_box_t {
  double x1, x2, y1, y2;
} glc_sub_pixel_region_box_t;

typedef struct _glc_point_t {
  double x, y;
} glc_point_t;

typedef struct _glc_matrix_t {
  double m[3][3];
} glc_matrix_t;

typedef struct _glc_texture {
  unsigned int name;
  unsigned int target;
  unsigned int format;
  glc_bool_t allocated;
  
  glc_filter_t filter;
  glc_bool_t repeat;
  
  int width;
  int height;

  double texcoord_width;
  double texcoord_height;

  glc_bool_t repeatable;
} glc_texture_t;

typedef struct glc_surface_backend {
  glc_surface_t *
  (*create_similar) (void *surface,
                     glc_format_name_t format_name,
                     int width,
                     int height);
  
  void
  (*destroy) (void *surface);
  
  glc_bool_t
  (*push_current) (void *surface,
                   glc_constraint_t constraint);

  void
  (*pop_current) (void *surface);

  glc_texture_t *
  (*get_texture) (void *surface);
  
  void
  (*realize) (void *surface);

  void
  (*show) (void *surface);
} glc_surface_backend_t;

struct _glc_surface {
  const glc_surface_backend_t *backend;

  glc_format_t *format;
  glc_texture_t *texture;
  long int status_mask;
  long int feature_mask;
  glc_filter_t filter;
  glc_bool_t repeat;
  glc_polyedge_t polyedge;
  glc_matrix_t *transform;
  glc_matrix_t *transforms;
  unsigned int n_transforms;
  glc_bool_t anti_aliasing;
  int width, height;
  glc_bool_t implicit_mask;
  glc_bool_t dirty;
  glc_region_box_t dirty_region;
  glc_bool_t requires_flipping;
  glc_gl_proc_address_list_t *proc_address;
  glc_programs_t *programs;
  glc_matrix_t *convolution;
  glc_bool_t clear_exterior;
  glc_bool_t programmatic;
};

typedef struct _glc_solid_t {
  glc_color_t color;
} glc_solid_t;

typedef struct _glc_linear_gradient_t {
  glc_colorpoint_t start;
  glc_colorpoint_t end;
} glc_linear_gradient_t;

typedef struct _glc_radial_gradient_t {
  glc_colorpoint_t start;
  glc_colorstop_t stop;
} glc_radial_gradient_t;

typedef union _glc_programmatic_attributes_t {
  glc_solid_t solid;
  glc_linear_gradient_t linear_gradient;
  glc_radial_gradient_t radial_gradient;
} glc_programmatic_attributes_t;

typedef struct _glc_programmatic_surface_t {
  glc_surface_t base;
  
  glc_programmatic_surface_type_t type;
  glc_programmatic_attributes_t attributes;
  glc_texture_t texture;
} glc_programmatic_surface_t;

typedef struct _glc_extension_map {
  char *name;
  int mask;
} glc_extension_map;

extern void __internal_linkage
glc_matrix_transform_point (glc_matrix_t *matrix,
                            glc_point_t *point);

extern void __internal_linkage
glc_matrix_transform_region (glc_matrix_t *matrix,
                             glc_region_box_t *region);
     
extern void __internal_linkage
glc_matrix_transform_sub_pixel_region (glc_matrix_t *matrix,
                                       glc_sub_pixel_region_box_t *region);

extern glc_status_t __internal_linkage
glc_matrix_invert (glc_matrix_t *matrix);

extern void __internal_linkage
glc_matrix_translate (glc_matrix_t *matrix,
                      double tx,
                      double ty);

extern glc_status_t __internal_linkage
glc_matrix_normalize (glc_matrix_t *matrix);

extern void __internal_linkage
glc_set_operator (glc_operator_t op);

extern void __internal_linkage
glc_intersect_region (glc_region_box_t *box1,
                      glc_region_box_t *box2,
                      glc_region_box_t *return_box);

extern void __internal_linkage
glc_union_region (glc_region_box_t *box1,
                  glc_region_box_t *box2,
                  glc_region_box_t *return_box);

extern void __internal_linkage
glc_intersect_sub_pixel_region (glc_sub_pixel_region_box_t *box1,
                                glc_sub_pixel_region_box_t *box2,
                                glc_sub_pixel_region_box_t *return_box);

extern void __internal_linkage
glc_union_sub_pixel_region (glc_sub_pixel_region_box_t *box1,
                            glc_sub_pixel_region_box_t *box2,
                            glc_sub_pixel_region_box_t *return_box);

extern unsigned int __internal_linkage
glc_get_gl_format_from_bpp (unsigned short bpp);

extern unsigned int __internal_linkage
glc_get_gl_data_type_from_bpp (unsigned short bpp);

extern long int __internal_linkage
glc_extensions_query (const char *extensions_string,
                      glc_extension_map *extensions_map);

extern glc_texture_t *__internal_linkage
glc_texture_generate (int width,
                      int height,
                      unsigned int texture_format,
                      long int target_mask);

extern void __internal_linkage
glc_texture_allocate (glc_texture_t *texture);

extern void __internal_linkage
glc_texture_destroy (glc_texture_t *texture);

extern void __internal_linkage
glc_texture_ensure_filter (glc_texture_t *texture,
                           glc_filter_t filter);

extern void __internal_linkage
glc_texture_ensure_repeat (glc_texture_t *texture,
                           glc_bool_t repeat);

extern void __internal_linkage
glc_texture_bind (glc_texture_t *texture);

extern void __internal_linkage
glc_texture_unbind (glc_texture_t *texture);


extern void __internal_linkage
glc_texture_copy_surface (glc_texture_t *texture,
                          glc_surface_t *surface,
                          glc_region_box_t *region);

extern void __internal_linkage
glc_surface_init (glc_surface_t *surface,
                  const glc_surface_backend_t *backend);

extern void __internal_linkage
glc_surface_push_transform (glc_surface_t *surface);

extern void __internal_linkage
glc_surface_pop_transform (glc_surface_t *surface);

extern void __internal_linkage
glc_surface_deinit (glc_surface_t *surface);

extern glc_surface_t * __internal_linkage
glc_surface_create_similar (glc_surface_t *templ,
                            glc_format_name_t format_name,
                            int width,
                            int height);

extern glc_texture_t *__internal_linkage
glc_surface_get_texture (glc_surface_t *surface);

extern void __internal_linkage
glc_surface_destory (glc_surface_t *surface);

extern glc_bool_t __internal_linkage
glc_surface_push_current (glc_surface_t *surface,
                          glc_constraint_t constraint);

extern void __internal_linkage
glc_surface_pop_current (glc_surface_t *surface);

extern void __internal_linkage
glc_surface_bounds (glc_surface_t *surface,
                    glc_region_box_t *box);

extern void __internal_linkage
glc_surface_enable_program (glc_program_type_t type,
                            glc_surface_t *surface,
                            glc_surface_t *src,
                            glc_surface_t *mask,
                            glc_texture_t *src_texture,
                            glc_texture_t *mask_texture);

extern void __internal_linkage
glc_surface_disable_program (glc_surface_t *surface);

extern void __internal_linkage
glc_surface_dirty (glc_surface_t *surface,
                   glc_region_box_t *region);

extern void __internal_linkage
glc_surface_status_add (glc_surface_t *surface, int flags);

extern void __internal_linkage
glc_surface_setup_environment (glc_surface_t *surface);

extern glc_status_t __internal_linkage
glc_status_pop_from_mask (long int *mask);

extern glc_format_t *__internal_linkage
glc_format_find (glc_format_t *formats,
                 int n_formats,
                 unsigned long mask,
                 const glc_format_t *templ,
                 int count);

extern glc_format_t *__internal_linkage
glc_format_find_standard (glc_format_t *formats,
                          int n_formats,
                          unsigned long options,
                          glc_format_name_t format_name);

extern glc_format_t *__internal_linkage
glc_format_find_sufficient_standard (glc_format_t *formats,
                                     int n_formats,
                                     unsigned long options,
                                     glc_format_name_t format_name);

void
glc_format_calculate_pixel_transfer_info (glc_format_t *format);

extern glc_program_type_t __internal_linkage
glc_program_type (glc_surface_t *dst,
                  glc_surface_t *src,
                  glc_surface_t *mask);

extern void __internal_linkage
glc_program_enable (glc_program_type_t type,
                    glc_surface_t *dst,
                    glc_surface_t *src,
                    glc_surface_t *mask,
                    glc_texture_t *src_texture,
                    glc_texture_t *mask_texture);

extern void __internal_linkage
glc_program_disable (glc_surface_t *dst);

extern void __internal_linkage
glc_programmatic_surface_setup (glc_surface_t *abstract_surface,
                                int width,
                                int height);

extern void __internal_linkage
glc_programmatic_surface_set_parameters (glc_gl_proc_address_list_t
                                         *proc_address,
                                         glc_programmatic_surface_t *surface);

#define MAXSHORT SHRT_MAX
#define MINSHORT SHRT_MIN

#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))

/* Fixed point updates from Carl Worth, USC, Information Sciences Institute */

#ifdef WIN32
typedef __int64 glc_fixed_32_32;
#else
#  if defined(__alpha__) || defined(__alpha) || \
      defined(ia64) || defined(__ia64__) || \
      defined(__sparc64__) || \
      defined(__s390x__) || \
      defined(x86_64) || defined (__x86_64__)
typedef long glc_fixed_32_32;
# else
#  if defined(__GNUC__) && \
    ((__GNUC__ > 2) || \
     ((__GNUC__ == 2) && defined(__GNUC_MINOR__) && (__GNUC_MINOR__ > 7)))
__extension__
#  endif
typedef long long int glc_fixed_32_32;
# endif
#endif

typedef uint32_t glc_fixed_1_31;
typedef uint32_t glc_fixed_1_16;
typedef int32_t glc_fixed_16_16;

/*
 * An unadorned "glc_fixed" is the same as glc_fixed_16_16, 
 * (since it's quite common in the code) 
 */
typedef glc_fixed_16_16 glc_fixed;

#define FIXED_BITS 16

#define FIXED_TO_INT(f) (int) ((f) >> FIXED_BITS)
#define INT_TO_FIXED(i) ((glc_fixed) (i) << FIXED_BITS)
#define FIXED_E ((glc_fixed) 1)
#define FIXED1 (INT_TO_FIXED(1))
#define FIXED1_MINUS_E (FIXED1 - FIXED_E)
#define FIXED_FRAC(f) ((f) & FIXED1_MINUS_E)
#define FIXED_FLOOR(f) ((f) & ~FIXED1_MINUS_E)
#define FIXED_CEIL(f) FIXED_FLOOR((f) + FIXED1_MINUS_E)

#define FIXED_FRACTION(f) ((f) & FIXED1_MINUS_E)
#define FIXED_MOD2(f) ((f) & (FIXED1 | FIXED1_MINUS_E))

#define FIXED_TO_DOUBLE(f) (((double) (f)) / 65536)
#define DOUBLE_TO_FIXED(f) ((int) ((f) * 65536))


/* Avoid unnecessary PLT entries.  */

slim_hidden_proto(glc_surface_set_transform)
slim_hidden_proto(glc_surface_set_convolution)
slim_hidden_proto(glc_surface_set_repeat)
slim_hidden_proto(glc_surface_set_filter)
slim_hidden_proto(glc_surface_set_polyedge)
slim_hidden_proto(glc_surface_get_width)
slim_hidden_proto(glc_surface_get_height)
slim_hidden_proto(glc_surface_realize)
slim_hidden_proto(glc_surface_show)
slim_hidden_proto(glc_surface_read_pixels)
slim_hidden_proto(glc_surface_draw_pixels)
slim_hidden_proto(glc_surface_get_status)
slim_hidden_proto(glc_surface_get_gl_texture)
slim_hidden_proto(glc_surface_gl_begin)
slim_hidden_proto(glc_surface_gl_end)
slim_hidden_proto(glc_surface_features)
slim_hidden_proto(glc_fill_rectangle)
slim_hidden_proto(glc_fill_rectangles)
slim_hidden_proto(glc_composite_trapezoids)
slim_hidden_proto(glc_color_trapezoids)
slim_hidden_proto(glc_composite_triangles)
slim_hidden_proto(glc_composite_tri_strip)
slim_hidden_proto(glc_composite_tri_fan)
slim_hidden_proto(glc_color_triangles)
slim_hidden_proto(glc_solid_create)
slim_hidden_proto(glc_linear_gradient_create)
slim_hidden_proto(glc_radial_gradient_create)
     
     
#ifdef LIBGLC_HAS_GLX_BACKEND

#include <GL/glx.h>

#include "glc_glxext.h"

#define GLC_GLX_FEATURE_TEXTURE_RECTANGLE_MASK    (1L << 0)
#define GLC_GLX_FEATURE_TEXTURE_NPOT_MASK         (1L << 1)
#define GLC_GLX_FEATURE_MULTISAMPLE_MASK          (1L << 2)
#define GLC_GLX_FEATURE_CLIENT_MULTISAMPLE_MASK   (1L << 3)
#define GLC_GLX_FEATURE_MULTISAMPLE_FILTER_MASK   (1L << 4)
#define GLC_GLX_FEATURE_ARB_VERTEX_PROGRAM_MASK   (1L << 5)
#define GLC_GLX_FEATURE_ARB_FRAGMENT_PROGRAM_MASK (1L << 6)
#define GLC_GLX_FEATURE_ATI_RENDER_TEXTURE_MASK   (1L << 7)
#define GLC_GLX_FEATURE_GLX13_MASK                (1L << 8)

typedef struct _glc_glx_surface glc_glx_surface_t;
typedef struct _glc_glx_screen_info_t glc_glx_screen_info_t;
typedef struct _glc_glx_display_info_t glc_glx_display_info_t;

typedef struct _glc_glx_static_proc_address_list_t {
  glc_proc_ptr_glx_get_fbconfigs_t glx_get_fbconfigs;
  glc_proc_ptr_glx_get_fbconfig_attrib_t glx_get_fbconfig_attrib;
  glc_proc_ptr_glx_get_visual_from_fbconfig_t glx_get_visual_from_fbconfig;
  glc_proc_ptr_glx_create_pbuffer_t glx_create_pbuffer;
  glc_proc_ptr_glx_destroy_pbuffer_t glx_destroy_pbuffer;
  glc_bool_t supported;
} glc_glx_static_proc_address_list_t;

typedef struct _glc_glx_proc_address_list_t {
  glc_proc_ptr_bind_tex_image_ati_t glx_bind_tex_image_ati;
  glc_proc_ptr_release_tex_image_ati_t glx_release_tex_image_ati;
  glc_bool_t supported;
} glc_glx_proc_address_list_t;

typedef struct _glc_glx_thread_info_t {
  glc_glx_display_info_t **displays;
  int n_displays;
} glc_glx_thread_info_t;

struct _glc_glx_display_info_t {
  glc_glx_thread_info_t *thread_info;
  Display *display;
  glc_glx_screen_info_t **screens;
  int n_screens;
};

typedef struct _glc_glx_context_info_t {
  glc_glx_surface_t *surface;
  glc_constraint_t constraint;
} glc_glx_context_info_t;

typedef struct _glc_glx_context_t {
  GLXContext context;
  XID id;
  glc_gl_proc_address_list_t gl_proc_address;
  glc_glx_proc_address_list_t glx_proc_address;
  GLXFBConfig fbconfig;
} glc_glx_context_t;

struct _glc_glx_screen_info_t {
  glc_glx_display_info_t *display_info;
  int screen;

  glc_format_t *formats;
  XID *format_ids;
  int n_formats;
  
  glc_glx_context_t **contexts;
  int n_contexts;
  
  glc_glx_context_info_t *context_stack;
  int context_stack_size;
  
  glc_glx_context_t root_context;
  GLXDrawable root_drawable;

  long int feature_mask;
  long int glx_feature_mask;
  long int texture_mask;

  glc_programs_t programs;
};

struct _glc_glx_surface {
  glc_surface_t base;
  
  glc_glx_screen_info_t *screen_info;
  glc_glx_context_t *context;
  GLXDrawable drawable;
  GLXDrawable pbuffer;
  glc_bool_t render_texture;
};

extern void __internal_linkage
glc_glx_query_extensions (glc_glx_screen_info_t *screen_info);

extern glc_glx_thread_info_t *__internal_linkage
glc_glx_thread_info_get (void);

extern glc_glx_display_info_t *__internal_linkage
glc_glx_display_info_get (Display *display);

extern glc_glx_screen_info_t *__internal_linkage
glc_glx_screen_info_get (Display *display,
                         int screen);

extern void *__internal_linkage
glc_glx_get_proc_address (const char *name);

extern glc_glx_context_t *__internal_linkage
glc_glx_context_get (glc_glx_screen_info_t *screen_info,
                     glc_format_t *format);

extern glc_glx_context_t *__internal_linkage
glc_glx_context_get_default (glc_glx_screen_info_t *screen_info);

extern void __internal_linkage
glc_glx_context_make_current (glc_glx_surface_t *surface);

extern glc_glx_surface_t *__internal_linkage
glc_glx_context_push_current (glc_glx_surface_t *surface,
                              glc_constraint_t constraint);

extern glc_glx_surface_t *__internal_linkage
glc_glx_context_pop_current (glc_glx_surface_t *surface);

extern void __internal_linkage
glc_glx_context_proc_address_lookup (glc_glx_context_t *context);

extern void __internal_linkage
glc_glx_query_formats (glc_glx_screen_info_t *screen_info);

extern GLXPbuffer __internal_linkage
glc_glx_pbuffer_create (Display *display,
                        GLXFBConfig fbconfig,
                        glc_texture_t *texture,
                        glc_bool_t render_texture);

extern void __internal_linkage
glc_glx_pbuffer_destroy (Display *display,
                         GLXPbuffer pbuffer);

/* Avoid unnecessary PLT entries.  */

slim_hidden_proto(glc_glx_initialize)
slim_hidden_proto(glc_glx_features)
slim_hidden_proto(glc_glx_find_format)
slim_hidden_proto(glc_glx_find_standard_format)
slim_hidden_proto(glc_glx_get_visual_info_from_format)
slim_hidden_proto(glc_glx_surface_create)
slim_hidden_proto(glc_glx_surface_create_for_window)

#endif /* LIBGLC_HAS_GLX_BACKEND */


#ifdef LIBGLC_HAS_AGL_BACKEND

#include <Carbon/Carbon.h>
#include <AGL/agl.h>

#define GLC_AGL_FEATURE_PBUFFER_MASK              (1L << 0)
#define GLC_AGL_FEATURE_TEXTURE_RECTANGLE_MASK    (1L << 1)
#define GLC_AGL_FEATURE_TEXTURE_NPOT_MASK         (1L << 2)
#define GLC_AGL_FEATURE_MULTISAMPLE_MASK          (1L << 3)
#define GLC_AGL_FEATURE_MULTISAMPLE_FILTER_MASK   (1L << 4)
#define GLC_AGL_FEATURE_ARB_VERTEX_PROGRAM_MASK   (1L << 5)
#define GLC_AGL_FEATURE_ARB_FRAGMENT_PROGRAM_MASK (1L << 6)

typedef struct _glc_agl_surface_t glc_agl_surface_t;

typedef struct _glc_agl_context_info_t {
  glc_agl_surface_t *surface;
  glc_constraint_t constraint;
} glc_agl_context_info_t;

typedef struct _glc_agl_context_t {
  AGLContext context;
  AGLPixelFormat pixel_format;
  glc_bool_t offscreen;
} glc_agl_context_t;

typedef struct _glc_agl_thread_info_t {
  glc_format_t *formats;
  AGLPixelFormat *format_ids;
  int n_formats;
  
  glc_agl_context_t **contexts;
  int n_contexts;
  
  glc_agl_context_info_t *context_stack;
  int context_stack_size;
  
  glc_agl_context_t root_context;

  long int feature_mask;
  long int agl_feature_mask;
  long int texture_mask;

  glc_programs_t programs;
} glc_agl_thread_info_t;

struct _glc_agl_surface_t {
  glc_surface_t base;
  
  glc_agl_thread_info_t *thread_info;
  glc_agl_context_t *context;
  AGLDrawable drawable;
  AGLPbuffer pbuffer;
  WindowRef window;
};

extern void __internal_linkage
glc_agl_query_extensions (glc_agl_thread_info_t *thread_info);

extern glc_agl_thread_info_t *__internal_linkage
glc_agl_thread_info_get (void);

extern glc_agl_context_t *__internal_linkage
glc_agl_context_get (glc_agl_thread_info_t *thread_info,
                     glc_format_t *format,
                     glc_bool_t offscreen);

extern void __internal_linkage
glc_agl_context_make_current (glc_agl_surface_t *surface);

extern glc_agl_surface_t *__internal_linkage
glc_agl_context_push_current (glc_agl_surface_t *surface,
                              glc_constraint_t constraint);

extern glc_agl_surface_t *__internal_linkage
glc_agl_context_pop_current (glc_agl_surface_t *surface);

extern void __internal_linkage
glc_agl_query_formats (glc_agl_thread_info_t *thread_info);

extern AGLPbuffer __internal_linkage
glc_agl_pbuffer_create (glc_texture_t *texture);

extern void __internal_linkage
glc_agl_pbuffer_bind (AGLPbuffer pbuffer,
                      AGLContext context,
                      glc_texture_t *texture,
                      glc_format_t *format);

extern void __internal_linkage
glc_agl_pbuffer_destroy (AGLPbuffer pbuffer);


/* Avoid unnecessary PLT entries.  */

slim_hidden_proto(glc_agl_initialize)
slim_hidden_proto(glc_agl_features)
slim_hidden_proto(glc_agl_find_format)
slim_hidden_proto(glc_agl_find_standard_format)
slim_hidden_proto(glc_agl_surface_create)
slim_hidden_proto(glc_agl_surface_create_for_window)

#endif /* LIBGLC_HAS_AGL_BACKEND */
     

#endif /* GLCINT_H_INCLUDED */
